UNIT LRLHighScores;

INTERFACE

USES DOSFileAccess,LRLRoutines,LRLSprites,StrUnit;

PROCEDURE LRLLoadHighScores;
PROCEDURE LRLShowHighScores;
FUNCTION LRLBestScore(Score: LONGINT): BOOLEAN;
PROCEDURE LRLInsertScore(Name: STRING; Score: LONGINT);
PROCEDURE LRLSaveHighScores;
FUNCTION LRLEnterName: STRING;

IMPLEMENTATION

CONST
	HighsFileName = 'LRL.HSR';
	HighsFileHeader: STRING[29] = 'Lode Runner Live High Scores'#26;

TYPE
	TSupers = RECORD
		Name: STRING[20];
		Score: LONGINT;
	END;

VAR
	MainScreen: POINTER;
	HighFrame: POINTER;
	HighTable: ARRAY[1..5] OF TSupers;

PROCEDURE LoadData;
VAR
	j: WORD;
BEGIN
	GETMEM(MainScreen,64004);
	GETMEM(HighFrame,45000);
	DFAFilePositionSet(ImageFile,LRLImagesFilePosition,DFASeekFromStart);
	DFAFileRead(ImageFile,MainScreen^,7940,j);
	DecompressRepByte(MainScreen^,HighFrame^,7940,j);
	DFAFileRead(ImageFile,MainScreen^,64004,j);
END;

PROCEDURE DisposeData;
BEGIN
	FREEMEM(MainScreen,64004);
	FREEMEM(HighFrame,45000);
END;

PROCEDURE LRLShowHighScores;
VAR
	p: POINTER;
	i: INTEGER;
	s: STRING;
BEGIN
	LRLLoadHighScores;
	GETMEM(p,768);
	DataFill(p^,768,0,0);
	Palette256Set(p^);
	FREEMEM(p,768);
	LoadData;
	ImagePut(LRLScreen^,MainScreen^,0,0,0,0,319,199);
	ImagePut(LRLScreen^,HighFrame^,6,50,0,0,319,199);
	FOR i:=1 TO 5 DO BEGIN
		ImageStringGet(CHR(i+48)+'. '+HighTable[i].Name,LRLFont^,LRLFontBuffer^,110);
		ImagePut(LRLScreen^,LRLFontBuffer^,55,85+i*17,8,0,319,199);
		STR(HighTable[i].Score,s);
		ImageStringGet(s,LRLFont^,LRLFontBuffer^,46);
		ImagePut(LRLScreen^,LRLFontBuffer^,260-ImageSizex(LRLFontBuffer^),85+i*17,8,0,319,199);
	END;
	ScreenApply(LRLScreen^);
	FadeTo(LRLMenuPalette);
	READKEY;
	FadeClear;
	ImageClear(LRLScreen^);
	ScreenApply(LRLScreen^);
	DisposeData;
END;

PROCEDURE LRLLoadHighScores;
VAR
	InFile: TDFAFileHandle;
	i,j: WORD;
	high: TSupers;
	dummy: STRING[30];
BEGIN
	high.name:='Lode Runner';
	DFAFileOpen(InFile,HighsFileName,DFAAccessReadWrite);
	IF DFALastResult(InFile)<>0 THEN BEGIN
		DFAFileCreate(InFile,HighsFileName,DFAAttributeArchive);
		DFAFileWrite(InFile,HighsFileHeader[1],29,i);
		FOR i:=1 TO 5 DO BEGIN
			high.score:=60000-i*10000;
			DFAFileWrite(InFile,high,SIZEOF(high),j);
		END;
	END;
	DFAFilePositionSet(InFile,0,DFASeekFromStart);
	DFAFileRead(InFile,dummy[1],29,j);
	IF (DFALastResult(InFile)<>0) OR (NOT DataIdentical(dummy[1],HighsFileHeader[1],29,0,0)) THEN BEGIN
		WRITELN('Error: Invalid file with high scores! (try to remove LRL.HSR file)');
		WRITELN('訡:  䠩  ४ठ! (஡ 㤠 䠩 LRL.HSR)');
		FHALT;
	END;
	DFAFileRead(InFile,HighTable,SIZEOF(TSupers)*5,j);
	DFAFileClose(InFile);
END;

PROCEDURE LRLSaveHighScores;
VAR
	InFile: TDFAFileHandle;
	i,j: WORD;
BEGIN
	DFAFileOpen(InFile,HighsFileName,DFAAccessReadWrite);
	DFAFilePositionSet(InFile,29,DFASeekFromStart);
	DFAFileWrite(InFile,HighTable,SIZEOF(TSupers)*5,j);
	DFAFileClose(InFile);
END;

FUNCTION LRLBestScore(Score: LONGINT): BOOLEAN;
VAR
	i: INTEGER;
BEGIN
	LRLBestScore:=TRUE;
	LRLLoadHighScores;
	i:=1;
	WHILE TRUE DO BEGIN
		IF Score>=HighTable[i].Score THEN EXIT;
		INC(i);
		IF i>5 THEN BEGIN
			LRLBestScore:=FALSE;
			EXIT;
		END;
	END;
END;

PROCEDURE LRLInsertScore(Name: STRING; Score: LONGINT);
VAR
	i,j: WORD;
BEGIN
	LRLLoadHighScores;
	i:=1;
	WHILE TRUE DO BEGIN
		IF Score>=HighTable[i].Score THEN BEGIN
			FOR j:=4 DOWNTO i DO BEGIN
				HighTable[j+1].Name:=HighTable[j].Name;
				HighTable[j+1].Score:=HighTable[j].Score;
			END;
			HighTable[i].Name:=Name;
			HighTable[i].Score:=Score;
			LRLSaveHighScores;
			EXIT;
		END;
		INC(i);
		IF i>5 THEN BEGIN
			EXIT;
		END;
	END;
END;

FUNCTION LRLEnterName: STRING;
VAR
	p: POINTER;
	i: INTEGER;
	RedrawName: BOOLEAN;
	Keypress: WORD;
	name: STRING;
BEGIN
	name:='';
	GETMEM(p,768);
	DataFill(p^,768,0,0);
	Palette256Set(p^);
	FREEMEM(p,768);
	ImageClear(LRLScreen^);
	ImagePut(LRLScreen^,LRLLogo^,3,3,0,0,319,199);
	ImageStringGet('Congratulations! You are in Top-Five!',LRLFont^,LRLFontBuffer^,110);
	ImagePut(LRLScreen^,LRLFontBuffer^,160-ImageSizex(LRLFontBuffer^) SHR 1,85,0,0,319,199);
	ImageStringGet('Enter your name below, Champ',LRLFont^,LRLFontBuffer^,111);
	ImagePut(LRLScreen^,LRLFontBuffer^,160-ImageSizex(LRLFontBuffer^) SHR 1,110,0,0,319,199);
	ImageStringGet('---------------------------',LRLFont^,LRLFontBuffer^,100);
	ImagePut(LRLScreen^,LRLFontBuffer^,160-ImageSizex(LRLFontBuffer^) SHR 1,155,0,0,319,199);
	ScreenApply(LRLScreen^);
	FadeTo(LRLMenuPalette);
	RedrawName:=TRUE;
	REPEAT
		IF RedrawName=TRUE THEN BEGIN
			ImageFill(LRLFontBuffer^,320,20,0);
			ImagePut(LRLScreen^,LRLFontBuffer^,0,140,0,0,319,199);
			ImageStringGet(name,LRLFont^,LRLFontBuffer^,100);
			ImagePut(LRLScreen^,LRLFontBuffer^,160-ImageSizex(LRLFontBuffer^) SHR 1,140,0,0,319,199);
			ScreenApply(LRLScreen^);
			RedrawName:=FALSE;
		END;
		Keypress:=READKEY;
		IF (LO(Keypress)=8) AND (LENGTH(name)>0) THEN BEGIN
			name[0]:=CHAR(ORD(name[0])-1);
			RedrawName:=TRUE;
		END;
		IF (LO(Keypress)>31) AND (LENGTH(name)<20) THEN BEGIN
			name:=name+CHAR(LO(Keypress));
			RedrawName:=TRUE;
		END;
	UNTIL LO(Keypress)=13;
	FadeClear;
	name:=StringTrimAll(name,' ');
	IF LENGTH(name)=0 THEN
		name:='Anonymous';
	LRLEnterName:=name;
END;

END.